local addonName, addon = ...

--[[----------------------------------------------------------------------------
Frame
------------------------------------------------------------------------------]]
local compass = CreateFrame('Frame', nil, UIParent)
compass:SetPoint('CENTER')
compass:SetMovable(true)
compass:SetSize(1, 1)

local suffix = GetLocale()
if suffix == 'koKR' then
	suffix = "_" .. suffix
else
	suffix = ""
end

local background = compass:CreateTexture(nil, 'BACKGROUND')
background:SetTexture([[Interface\Addons\]] .. addonName .. [[\Compass]] .. suffix)
background:SetAllPoints(compass)

local pointer = compass:CreateTexture(nil, 'ARTWORK')
pointer:SetTexture([[Interface\Addons\]] .. addonName .. [[\Pointer]])

--[[----------------------------------------------------------------------------
Helpers
------------------------------------------------------------------------------]]
local ADJ_FACTOR, DEG_TO_COORD, PI, SCREEN_HEIGHT, SCREEN_WIDTH = 1 / math.rad(720), 1 / 1440, math.rad(180)
local background, shown, timer, adjCoord, currentFacing, interval = background, true, 0
local hideConditions = { ['disable'] = 0, ['false'] = 0, ['hide'] = 0, ['nil'] = 0, ["0"] = 0, [0] = 0 }
local settings, stateDriver

local GetPlayerFacing, GetRotation, SetTexCoord = GetPlayerFacing, background.GetRotation, background.SetTexCoord
local math_atan2, MinimapCompassTexture = math.atan2, MinimapCompassTexture

local strata_layers = LibStub('LibDisplayAssist-1.3').StrataLayers

local function GetOffset(asPercent)
	local anchor, x, y = settings.anchor
	local L1, B1, W1, H1 = UIParent:GetRect()
	local L2, B2, W2, H2 = compass:GetRect()
	if anchor == 'TOP' then
		x, y = L2 - L1 + (W2 - W1) * 0.5, B2 + H2 - B1 - H1
	elseif anchor == 'BOTTOM' then
		x, y = L2 - L1 + (W2 - W1) * 0.5, B2 - B1
	elseif anchor == 'LEFT' then
		x, y = L2 - L1, B2 - B1 + (H2 - H1) * 0.5
	elseif anchor == 'RIGHT' then
		x, y = L2 + W2 - L1 - W1, B2 - B1 + (H2 - H1) * 0.5
	elseif anchor == 'TOPLEFT' then
		x, y = L2 - L1, B2 + H2 - B1 - H1
	elseif anchor == 'TOPRIGHT' then
		x, y = L2 + W2 - L1 - W1, B2 + H2 - B1 - H1
	elseif anchor == 'BOTTOMLEFT' then
		x, y = L2 - L1, B2 - B1
	elseif anchor == 'BOTTOMRIGHT' then
		x, y = L2 + W2 - L1 - W1, B2 - B1
	else
		x, y = L2 - L1 + (W2 - W1) * 0.5, B2 - B1 + (H2 - H1) * 0.5
	end
	if asPercent then
		return x / SCREEN_WIDTH, y / SCREEN_HEIGHT
	end
	return x, y
end

local function UpdateAlpha(self, value)
	value = hideConditions[value] or tonumber(value) or settings.alpha
	if value == 0 then
		if shown then
			shown = false
			self:Hide()
		end
	else
		self:SetAlpha(value)
		if not shown then
			currentFacing, shown, timer = nil, true, 1
			self:Show()
		end
	end
end

local function UpdateProfile(event)
	settings = addon.db.profile
	if event then
		addon:ApplySettings()
	end
end

local function UpdateResolution(event, screenWidth, screenHeight)
	SCREEN_HEIGHT, SCREEN_WIDTH = screenHeight, screenWidth
	addon:ApplySettings()
end

--[[----------------------------------------------------------------------------
Global to addon
------------------------------------------------------------------------------]]
function addon:ApplySettings()
	adjCoord, currentFacing, interval, timer = settings.degrees * DEG_TO_COORD, nil, 1 / settings.frequency, 1
	background:SetVertexColor(settings.red, settings.green, settings.blue, 1)
	if settings.lock then
		compass.moving = nil
		compass:StopMovingOrSizing()
		compass:EnableMouse(false)
	else
		compass:EnableMouse(true)
	end
	compass:SetFrameStrata(strata_layers[settings.strata])
	compass:SetFrameLevel(settings.level)
	local clamp = settings.clamp
	compass:SetClampedToScreen(false)
	compass:SetClampRectInsets(clamp, -clamp, -clamp, clamp)
	compass:SetSize(settings.width * settings.scale, settings.height * settings.scale)
	compass:SetClampedToScreen(true)
	addon:SetOffset(settings.offsetX, settings.offsetY)
	pointer:SetVertexColor(settings.red, settings.green, settings.blue, 1)
	pointer:ClearAllPoints()
	if settings.flipFlop then
		pointer:SetTexCoord(0, 0.90625, 1, 0)
		pointer:SetPoint('TOP', compass, 'BOTTOM', 0, 0)
	else
		pointer:SetTexCoord(0, 0.90625, 0, 1)
		pointer:SetPoint('BOTTOM', compass, 'TOP', 0, 0)
	end
	local scale = (settings.height + 28) / 56 * settings.scale
	pointer:SetSize(14.5 * scale, 8 * scale)
	stateDriver:SetParameters(settings.parameters, true)
end

function addon:SaveOffset()
	settings.offsetX, settings.offsetY = GetOffset(true)
end

function addon:SetOffset(x, y)
	compass:ClearAllPoints()
	compass:SetPoint(settings.anchor, UIParent, settings.anchor, x * SCREEN_WIDTH, y * SCREEN_HEIGHT)
end

--[[----------------------------------------------------------------------------
Scripts
------------------------------------------------------------------------------]]
compass:SetScript('OnMouseDown', function(self, button)
	if button == 'LeftButton' then
		self.moving = true
		self:SetFrameStrata('TOOLTIP')
		self:StartMoving()
	end
end)

compass:SetScript('OnMouseUp', function(self, button)
	if button == 'LeftButton' then
		self:StopMovingOrSizing()
		self:SetFrameStrata(strata_layers[settings.strata])
		self.moving = nil
		addon:SaveOffset()
	elseif button == 'RightButton' and not self.moving then
		addon.config()
	end
end)

local function OnUpdate_Facing(self, elapsed)
	timer = timer + elapsed
	if timer < interval then return end
	timer = 0
	local facing = GetPlayerFacing() or 0
	if facing ~= currentFacing then
		local coord = (facing < PI and 0.5 or 1) - (facing * ADJ_FACTOR)
		SetTexCoord(background, coord - adjCoord, coord + adjCoord, 0.0625, 0.9375)
		currentFacing = facing
	end
end

local function OnUpdate_Rotate(self, elapsed)
	timer = timer + elapsed
	if timer < interval then return end
	timer = 0
	local facing = -GetRotation(MinimapCompassTexture) or 0
	if facing ~= currentFacing then
		local coord = (facing < PI and 0.5 or 1) - (facing * ADJ_FACTOR)
		SetTexCoord(background, coord - adjCoord, coord + adjCoord, 0.0625, 0.9375)
		currentFacing = facing
	end
end

--[[----------------------------------------------------------------------------
Evaluate state
------------------------------------------------------------------------------]]
local function EvaluateState()
	local onUpdate
	if GetPlayerFacing() then
		onUpdate = OnUpdate_Facing
	elseif GetCVarBool('rotateMinimap') and Minimap:IsShown() then
		onUpdate = OnUpdate_Rotate
	end
	compass:SetScript('OnUpdate', onUpdate)
	stateDriver:SetEnabled(onUpdate)
	if onUpdate then
		UpdateAlpha(compass, stateDriver:GetState())
	else
		UpdateAlpha(compass, 0)
	end
end

--[[----------------------------------------------------------------------------
Initialize
------------------------------------------------------------------------------]]
LibStub('LibDisplayAssist-1.3').Register(addon, function(_, ...)				-- Runs on first frame update to ensure proper sizing
	LibStub('LibDisplayAssist-1.3').Unregister(addon)
	stateDriver = LibStub('LibStateDriver-1.3').New(false, "SetCallback", UpdateAlpha, "SetObject", compass)
	addon.db = LibStub('AceDB-3.0'):New(addonName .. "Settings", { profile = addon.defaultSettings }, 'Default')
	UpdateProfile()
	UpdateResolution(nil, ...)
	stateDriver:SetEnabled(true)

	Minimap:HookScript('OnHide', EvaluateState)
	Minimap:HookScript('OnShow', EvaluateState)
	hooksecurefunc(_G, 'Minimap_UpdateRotationSetting', EvaluateState)
	compass:SetScript('OnEvent', EvaluateState)
	compass:RegisterEvent('PLAYER_ENTERING_WORLD')
	EvaluateState()

	CONFIGMODE_CALLBACKS[addonName] = addon.ConfigMode
	addon.db.RegisterCallback(addon, 'OnProfileChanged', UpdateProfile)
	addon.db.RegisterCallback(addon, 'OnProfileCopied', UpdateProfile)
	addon.db.RegisterCallback(addon, 'OnProfileReset', UpdateProfile)
	LibStub('LibDisplayAssist-1.3').Register(addon, UpdateResolution)
end)

LibStub('LibDisplayAssist-1.3').RegisterLogout(addon, function()				-- Runs on PLAYER_LOGOUT
	addon:SaveOffset()
end)
