local addonName, addon, L = LibStub('LibOptionsAssist-1.1').FetchData()

--[[-----------------------------------------------------------------------------
Helpers
-------------------------------------------------------------------------------]]
local function Get(info)
	return addon.db.profile[info[#info]]
end

local function GetColor(info)
	local settings = addon.db.profile
	return settings.red, settings.green, settings.blue
end

local function Set(info, value)
	addon.db.profile[info[#info]] = value
	addon:ApplySettings()
end

local function SetAutoRotate(info, value)
	addon.db.profile[info[#info]] = value
	addon.EvaluateState()
end

local function SetColor(info, red, green, blue)
	local settings = addon.db.profile
	settings.red, settings.green, settings.blue = red, green, blue
	addon:ApplySettings()
end

--[[-----------------------------------------------------------------------------
Options
-------------------------------------------------------------------------------]]
local options = {
	type = 'group', childGroups = 'tab',
	get = Get,
	set = Set,
	args = {
		general = {
			type = 'group',
			name = L["General"],
			desc = L["General settings."],
			args = {
				alpha = {
					type = 'range',
					name = L["Alpha, Base"],
					desc = L["Set the base alpha level to use for the compass."],
					min = 0, max = 1, step = 0.01
				},
				color = {
					type = 'color',
					name = L["Color"],
					desc = L["Change the color of the compass."],
					width = 'half',
					get = GetColor,
					set = SetColor
				},
				degrees = {
					type = 'range',
					name = L["Degrees"],
					desc = L["Set the displayed degree range for the compass."],
					min = 135, max = 360, step = 15
				},
				flipFlop = {
					type = 'toggle',
					name = L["Flip Flop"],
					desc = L["Determines if the compass pointer should be placed on the bottom instead of on the top."],
					width = 'half'
				},
				frequency = {
					type = 'range',
					name = L["Update Frequency"],
					desc = L["Set the number of times to update the compass each second."],
					min = 10, max = 120, step = 1
				},
				height = {
					type = 'range',
					name = L["Height"],
					desc = L["Set the height of the compass."],
					min = 7, max = 28, step = 0.01
				},
				scale = {
					type = 'range',
					name = L["Scale"],
					desc = L["Set the scale of the compass."],
					min = 0.5, max = 1.5, step = 0.01
				},
				width = {
					type = 'range',
					name = L["Width"],
					desc = L["Set the width of the compass."],
					min = 96, max = 384, step = 0.01
				},
				parameters = {
					type = 'input', multiline = 5,
					order = -2,
					name = L["Alpha, Parameters"],
					desc = L["Enter macro-style parameters that will be evaluated to set the alpha level of the compass, falling back to the base alpha as needed.\n\nExample: [combat][group] 0.5"],
					width = 'full'
				}
			}
		},
		position = {
			type = 'group',
			name = L["Position"],
			desc = L["Position related settings."],
			args= {
				anchor = {
					type = 'select',
					name = L["Anchor"],
					desc = L["Choose the screen region for positioning to be relative to."],
					values = LibStub('LibDisplayAssist-1.3').AnchorPoints,
					set = function(info, value)
						addon.db.profile.anchor = value
						addon:SaveOffset()
					end
				},
				clamp = {
					type = 'range',
					name = L["Screen Clamp"],
					desc = L["Set how much off-screen movement to allow."],
					min = 0, max = 5, step = 1
				},
				level = {
					type = 'range',
					name = L["Level"],
					desc = L["Set the frame level."],
					min = 0, max = 127, step = 1
				},
				lock = {
					type = 'toggle',
					name = L["Lock"],
					desc = L["Determines whether or not the compass can be repositioned with the mouse."],
					width = 'half'
				},
				strata = {
					type = 'select',
					name = L["Strata"],
					desc = L["Choose the frame strata."],
					values = LibStub('LibDisplayAssist-1.3').StrataLayers
				},
				reset = {
					type = 'execute',
					order = -1,
					name = L["Reset"],
					desc = L["Reset position to the current anchor."],
					func = function()
						addon:SetOffset(0, 0)
						addon:SaveOffset()
					end
				},
			}
		}
	}
}

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(addonName, options)

local profileOptions = LibStub('AceDBOptions-3.0'):GetOptionsTable(addon.db)
options.args.profiles = profileOptions
profileOptions.order = -1
profileOptions.get = false
profileOptions.set = false

addon.config:AssignOptions(addonName)
addon.config:SetDesc(L["These options allow you to change the appearance and behavior of %s."]:format(addonName))
addon.config:SetInfo(L["Version: %s"]:format(GetAddOnMetadata(addonName, 'Version')))
